import Page from './CPDFPage'
import TextAnnotation from './CPDFTextAnnotation'
import { AnnotationType } from './type';

export default class CPDFAnnotations {
  doc: number; // Document对象指针
  messageHandler: any;
  pages: Page[];
  static TextAnnotation = TextAnnotation

  constructor (options: {
    doc: number;
    messageHandler: any;
    pages: Page[];
  }) {
    this.doc = options.doc
    this.messageHandler = options.messageHandler
    this.pages = options.pages
  };

  async createAnnot (annotation: any, options?: object) {
    let pagePtr = await this.initPage(0)
    let annotPtr = await this.messageHandler.sendWithPromise('CreateAnnot', {
      doc: this.doc,
      page: pagePtr,
      type: annotation.type
    })

    await this.messageHandler.send('SetAnnotParam', {
      annot: annotPtr,
      page: pagePtr
    })
    return annotPtr
  };

  async initPage (pageIndex: number) {
    if (!this.pages[pageIndex]) {
      this.pages[pageIndex] = await this.messageHandler.sendWithPromise('InitPage', {
        doc: this.doc,
        pageIndex
      })
    }
    return this.pages[pageIndex]
  }

  render(annotation: any) {
    switch (annotation.type) {
      case AnnotationType.text:
        return new TextAnnotation(annotation)
      default:
        return new TextAnnotation(annotation)
    }
  }

  addAnnotation(annotation: any) {
    const annotations = Array.isArray(annotation) ? annotation : [annotation]
    return this.addAnnotations(annotations)
  }

  addAnnotations(annotations: any) {
    for (let i =0; i < annotations.length; i++) {
      const annotation = annotations[i]
      if (annotation.status !== 1) {
        const ctx = this.pages[annotation.pageNumber].annotationCanvas!.getContext('2d')!
        annotation.draw(ctx)
      }
    }
  }

  async deleteAnnotation(annotation: any) {
    const annotations = Array.isArray(annotation) ? annotation : [annotation]
    return this.deleteAnnotations(annotations)
  }

  async deleteAnnotations(annotations: any) {
    return await this.messageHandler.send('RemoveAnnot', annotations)
  }
};
