// @ts-nocheck
import CPDFAnnotation from "./CPDFAnnotation";
import Page from './CPDFPage';

export default class CPDFFreetextAnnotation extends CPDFAnnotation {
  private _annotFont: any;
  private _annotFontDa: any;
  private _freeTextType: any;
  private _freeTextContent: any;
  private _freeTextBgColor: any;
  private _freeTextBorderColor: any;
  pages: Page[]

  async initAnnot (pageIndex: number) {
    if (!this.pages[pageIndex]) {
      this.pages[pageIndex] = await this.messageHandler.sendWithPromise('InitAnnot', {
        page: this.page,
        pageIndex
      }) as any
    }
    return this.pages[pageIndex]
  }

  get annotFont () {
    return this._annotFont;
  }

  set annotFont (data: any) {
    this.setAnnotFont(data);
  }

  async getAnnotFont () {
    let annotPtr = await this.initAnnot(0)
    this._annotFont = await this.messageHandler.sendWithPromise('GetAnnotFont', {
      annot: annotPtr
    })
    return this._annotFont
  }

  async setAnnotFont (data: any) {
    const { fontName, fontSize, R, G, B, A } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotFont', {
      annot: annotPtr,
      fontName,
      fontSize,
      R,
      G,
      B,
      A
    })
  }

  get annotFontDa () {
    return this._annotFontDa;
  }

  set annotFontDa (data: any) {
    this.setAnnotFontDa(data);
  }

  async getAnnotFontDa () {
    let annotPtr = await this.initAnnot(0)
    this._annotFontDa = await this.messageHandler.sendWithPromise('GetAnnotFontDa', {
      annot: annotPtr
    })
    return this._annotFontDa
  }

  async setAnnotFontDa (data: any) {
    const { fontName, fontSize, R, G, B } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotFontDa', {
      annot: annotPtr,
      fontName,
      fontSize,
      R,
      G,
      B
    })
  }

  get freeTextType () {
    return this._freeTextType;
  }

  set freeTextType (data: any) {
    this.setFreeTextType(data);
  }

  async getFreeTextType () {
    let annotPtr = await this.initAnnot(0)
    this._freeTextType = await this.messageHandler.sendWithPromise('GetFreeTextType', {
      annot: annotPtr
    })
    return this._freeTextType
  }

  async setFreeTextType (data: any) {
    const { freeTextType } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetFreeTextType', {
      annot: annotPtr,
      freeTextType
    })
  }

  async getFreeTextFont () {
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('GetFreeTextFont', {
      annot: annotPtr
    })
  }

  get freeTextContent () {
    return this._freeTextContent;
  }

  set freeTextContent (data: any) {
    this.setFreeTextContent(data);
  }

  async getFreeTextContent () {
    let annotPtr = await this.initAnnot(0)
    this._freeTextContent = await this.messageHandler.sendWithPromise('GetFreeTextContent', {
      annot: annotPtr
    })
    return this._freeTextContent
  }

  async setFreeTextContent (data: any) {
    const { content } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetFreeTextContent', {
      annot: annotPtr,
      content
    })
  }

  async setFreeText (data: any) {
    const { fontName, fontSize, R, G, B, textAlignment } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotFontDa', {
      annot: annotPtr,
      fontName,
      fontSize,
      R,
      G,
      B,
      textAlignment
    })
  }

  get freeTextBgColor () {
    return this._freeTextBgColor;
  }

  set freeTextBgColor (data: any) {
    this.setFreeTextBgColor(data);
  }

  async getFreeTextBgColor () {
    let annotPtr = await this.initAnnot(0)
    this._freeTextBgColor = await this.messageHandler.sendWithPromise('GetFreeTextBgColor', {
      annot: annotPtr
    })
    return this._freeTextBgColor
  }

  async setFreeTextBgColor (data: any) {
    const { R, G, B } = data
    if (R === 0 && G === 0 && B === 0) {
      this.clearFreeTextBgColor()
      return
    }
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetFreeTextBgColor', {
      annot: annotPtr,
      R,
      G,
      B
    })
  }

  async clearFreeTextBgColor () {
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('ClearFreeTextBgColor', {
      annot: annotPtr
    })
  }

  get freeTextBorderColor () {
    return this._freeTextBorderColor;
  }

  set freeTextBorderColor (data: any) {
    this.setFreeTextBorderColor(data);
  }

  async getFreeTextBorderColor () {
    let annotPtr = await this.initAnnot(0)
    this._freeTextBorderColor = await this.messageHandler.sendWithPromise('GetFreeTextBorderColor', {
      annot: annotPtr
    })
    return this._freeTextBorderColor
  }

  async setFreeTextBorderColor (data: any) {
    const { R, G, B } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetFreeTextBorderColor', {
      annot: annotPtr,
      R,
      G,
      B
    })
  }

  async getFreetextTextRect () {
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('GetFreetextTextRect', {
      page: this.page,
      annot: annotPtr
    })
  }
}
