// @ts-nocheck
import CPDFAnnotation from "./CPDFAnnotation";

export default class CPDFShapeAnnotation extends CPDFAnnotation {
  private _annotBorderWidth: any;
  private _annotBorderRGBColor: any;
  private _annotBorderStyle: any;
  private _annotFilledRGBColor: any;
  private _annotFilledTransparency: any;
  private _annotBgRGBColor: any;

  async initAnnot (pageIndex: number) {
    if (!this.pages[pageIndex]) {
      this.pages[pageIndex] = await this.messageHandler.sendWithPromise('InitAnnot', {
        page: this.page,
        pageIndex
      })
    }
    return this.pages[pageIndex]
  }

  get annotBorderWidth () {
    return this._annotBorderWidth;
  }

  set annotBorderWidth (data: any) {
    this.setAnnotBorderWidth(data);
  }

  async getAnnotBorderWidth () {
    let annotPtr = await this.initAnnot(0)
    this._annotBorderWidth = await this.messageHandler.sendWithPromise('GetAnnotBorderWidth', {
      annot: annotPtr
    })
    return this._annotBorderWidth
  }

  async setAnnotBorderWidth (data: any) {
    const { width } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotBorderWidth', {
      annot: annotPtr,
      width
    })
  }

  get annotBorderRGBColor () {
    return this._annotBorderRGBColor;
  }

  set annotBorderRGBColor (data: any) {
    this.setAnnotBorderRGBColor(data);
  }

  async getAnnotBorderRGBColor () {
    let annotPtr = await this.initAnnot(0)
    this._annotBorderRGBColor = await this.messageHandler.sendWithPromise('GetAnnotBorderRGBColor', {
      annot: annotPtr
    })
    return this._annotBorderRGBColor
  }

  async setAnnotBorderRGBColor (data: any) {
    const { R, G, B } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotBorderRGBColor', {
      annot: annotPtr,
      R,
      G,
      B
    })
  }

  async getAnnotBorderStyleDashSize () {
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('GetAnnotBorderStyleDashSize', {
      annot: annotPtr
    })
  }

  get annotBorderStyle () {
    return this._annotBorderStyle;
  }

  set annotBorderStyle (data: any) {
    this.setAnnotBorderStyle(data);
  }

  async getAnnotBorderStyle () {
    let annotPtr = await this.initAnnot(0)
    this._annotBorderStyle = await this.messageHandler.sendWithPromise('GetAnnotBorderStyle', {
      annot: annotPtr
    })
    return this._annotBorderStyle
  }

  async setAnnotBorderStyle (data: any) {
    const { borderStyle, border, dash, dashSize } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotBorderStyle', {
      annot: annotPtr,
      borderStyle,
      border,
      dash,
      dashSize
    })
  }

  get annotFilledRGBColor () {
    return this._annotFilledRGBColor;
  }

  set annotFilledRGBColor (data: any) {
    this.setAnnotFilledRGBColor(data);
  }

  async getAnnotFilledRGBColor () {
    let annotPtr = await this.initAnnot(0)
    this._annotFilledRGBColor = await this.messageHandler.sendWithPromise('GetAnnotFilledRGBColor', {
      annot: annotPtr
    })
    return this._annotFilledRGBColor
  }

  async setAnnotFilledRGBColor (data: any) {
    const { R, G, B } = data
    if (R === 0 && G === 0 && B === 0) {
      this.clearAnnotFilledRGBColor()
      return
    }
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotFilledRGBColor', {
      annot: annotPtr,
      R,
      G,
      B
    })
  }

  async clearAnnotFilledRGBColor () {
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('ClearAnnotFilledRGBColor', {
      annot: annotPtr
    })
  }

  get annotFilledTransparency () {
    return this._annotFilledTransparency;
  }

  set annotFilledTransparency (data: any) {
    this.setAnnotFilledRGBColor(data);
  }

  async getAnnotFilledTransparency () {
    let annotPtr = await this.initAnnot(0)
    this._annotFilledTransparency = await this.messageHandler.sendWithPromise('GetAnnotFilledTransparency', {
      annot: annotPtr
    })
    return this._annotFilledTransparency
  }

  async setAnnotFilledTransparency (data: any) {
    const { transparency } = data
    if (transparency === 0) {
      this.clearAnnotFilledTransparency()
      return
    }
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAnnotFilledTransparency', {
      annot: annotPtr,
      transparency
    })
  }

  async clearAnnotFilledTransparency () {
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('ClearAnnotFilledTransparency', {
      annot: annotPtr
    })
  }

  get annotBgRGBColor () {
    return this._annotBgRGBColor;
  }

  set annotBgRGBColor (data: any) {
    this.setAnnotFilledRGBColor(data);
  }

  async getAannotBgRGBColor () {
    let annotPtr = await this.initAnnot(0)
    this._annotBgRGBColor = await this.messageHandler.sendWithPromise('GetAannotBgRGBColor', {
      annot: annotPtr
    })
    return this._annotBgRGBColor
  }

  async setAannotBgRGBColor (data: any) {
    const { R, G, B } = data
    let annotPtr = await this.initAnnot(0)
    return await this.messageHandler.sendWithPromise('SetAannotBgRGBColor', {
      annot: annotPtr,
      R,
      G,
      B
    })
  }
}
