import CPDFAnnotation from "./CPDFAnnotation";
import MessageHandler from "./MessageHandler"

export default class CPDFTextAnnotation extends CPDFAnnotation {
  constructor(options: {
    messageHandler: MessageHandler;
    annotPtr: number;
    ptr: number;
    rect: {
      left: number;
      top: number;
      right: number;
      bottom: number;
    },
    color: {
      R: number;
      G: number;
      B: number;
    };
    pageNumber: number;
    hidden: any;
    type: any;
    content: string;
    transparency?: number;
    createDate?: Date;
    modifyDate?: Date
  }) {
    super(options)
  }

  draw(ctx: CanvasRenderingContext2D) {
    ctx.save()
    ctx.translate(this.rect.left, this.rect.top)

    ctx.fillStyle = 'white'
    ctx.fillRect(0, 0, 24, 24)
    // M2.4 3.6 H21.6 V18 H12 L6 21.6 V18 H2.4 V3.6Z
    ctx.fillStyle = this.color; // 描边颜色使用当前颜色
    ctx.lineWidth = 0; // 设置线宽

    // 开始路径
    ctx.beginPath();

    // 移动到起始点 (M2.4 3.6)
    ctx.moveTo(2.4, 3.6);

    // 绘制直线 (H21.6)
    ctx.lineTo(21.6, 3.6);

    // 绘制直线 (V18)
    ctx.lineTo(21.6, 18);

    // 绘制直线 (H12)
    ctx.lineTo(12, 18);

    // 绘制直线 (L6 21.6)
    ctx.lineTo(6, 21.6);

    // 绘制直线 (V18)
    ctx.lineTo(6, 18);

    // 绘制直线 (H2.4)
    ctx.lineTo(2.4, 18);
    
    // 绘制直线 (V3.6)
    ctx.lineTo(2.4, 3.6);

    // 封闭路径，将最后一个点连接到第一个点
    ctx.closePath();

    // 描边路径
    ctx.fill();

    // 绘制白色矩形
    ctx.fillStyle = 'white'
    ctx.fillRect(6, 7.2, 7.2, 1.8)
    ctx.fillRect(6, 11.4, 12, 1.8)

    ctx.restore()
  }
}
