function throwError(error: any): void {
  console.error(error)
  throw error
}

type EventCallback = (...params: any[]) => any

export default class EventManager {

  private _events: Record<string, { handler: EventCallback }[]>

  constructor () {
    this._events = {}
  }

  addEvent(eventName: string, fn: EventCallback, options?: {
    once: boolean;
  }) {
    const self = this;
    if (!eventName || !fn || "function" !== typeof fn) {
      return this
    }
    this._events || (this._events = {});
    this._events[eventName] || (this._events[eventName] = []);

    let rawFn = fn
    options && options.once && (rawFn = function () {
      for (var argus = [], i = 0; i < arguments.length; i++) {
        argus[i] = arguments[i];
      }
      self.removeEvent(eventName, fn);
      fn.apply(self)
    });

    this._events[eventName].push({
      handler: rawFn
    });
    return this
  }

  trigger(eventName: string, data?: any) {
    if (!this._events || !this._events[eventName] || this._events[eventName].length === 0) {
      return this
    }
    const self = this

    this._events[eventName].map(function (event) {
      return event.handler
    }).forEach(function (handler) {
      try {
        data ? handler.apply(self, [data]) : handler.call(self)
      } catch (error) {
        throwError(error)
      }
    });
    return this
  }

  flattenArrays() {
    var totalLength = 0;
    for (var i = 0, length = arguments.length; i < length; i++) {
      totalLength += arguments[i].length;
    }
    var result = Array(totalLength);
    var currentIndex = 0;
    for (i = 0; i < length; i++) {
      var currentArray = arguments[i];
      for (var j = 0, arrayLength = currentArray.length; j < arrayLength; j++, currentIndex++) {
        result[currentIndex] = currentArray[j];
      }
    }
    return result;
  }

  removeEvent(eventName: string, fn?: EventCallback) {
    if (!this._events) {
      return this
    }
    eventName && (this._events = {})

    if (!this._events[eventName]) {
      return this
    }

    this._events[eventName] = fn ? this._events[eventName].filter(function (event) {
      return event.handler !== fn
    }) : [];
    return this
  }


}
