importScripts("/lib/ComPDFkit.js")

import { parseAdobePDFTimestamp, roundToDecimalPlaces } from '../utils';

let ComPDFKitJS = {}
let DataArray = []
let PageSize = {}
let Rect = {}
let RGBColor = {}
let FontDa = {}

import MessageHandler from "../MessageHandler.ts"

async function sleep(delay) {
  await new Promise((resolve) => setTimeout(resolve, delay))
}

class CPDFWorker {
  static async setup(doc) {
    this.doc = doc
    const messageHandler = new MessageHandler('worker', 'main', self)
    this.MessageHandler = messageHandler

    let testMessageProcessed = false
    
    messageHandler.on("test", function (data) {
      if (testMessageProcessed) {
        return; // we already processed 'test' message once
      }
      testMessageProcessed = true;

      // Ensure that `TypedArray`s can be sent to the worker.
      messageHandler.send("test", data instanceof Uint8Array);
    });

    messageHandler.send("ready", null)

    messageHandler.send("initDoc", { doc })

    messageHandler.on("Ready", function (data) {
      return CPDFWorker.createDocumentHandler(self)
    })
  }

  static createDocumentHandler(port) {
    const messageHandler = this.MessageHandler
    
    messageHandler.on('LoadFile', (data) => {
      ComPDFKitJS.opened_files = []
      ComPDFKitJS.opened_files[0] = data.buffer
      return true
    })

    messageHandler.on('LoadDocumentByStream', (data) => {
      const { doc, fileId, length } = data
      return Module._LoadDocumentByStream(doc, fileId, length)
    })

    messageHandler.on('GetPageCount', (data) => {
      const { doc } = data
      return Module._GetPageCount(doc)
    })

    messageHandler.on('GetPageSize', (data) => {
      const { doc, pageIndex } = data

      PageSize.Width = 0
      PageSize.Height = 0

      Module._GetPageSizeForIndex(doc, pageIndex)
      const page = Module._GetPage(doc, pageIndex)
      return {
        page,
        width: PageSize.Width,
        height: PageSize.Height
      }
    })

    messageHandler.on('GetAnnot', (data) => {
      const { page } = data
      const annotCount = Module._GetAnnotCount(page)
      const annotations = []
      for (let i = 0; i < annotCount; i++) {
        const annotPtr = Module._InitAnnot(page, i)
        const type = Module._GetAnnotType(annotPtr)
        const annotation = {
          annotPtr,
          type,
          ptr: page
        }
        annotations.push(annotation)
      }

      return {
        annotCount,
        annotations
      }
    })

    messageHandler.on('GetAnnotText', (data) => {
      const { page, annotPtr } = data

      Rect = {}
      Module._GetAnnotRect(page, annotPtr)
      const rect = {
        left: roundToDecimalPlaces(Rect.Left),
        top: roundToDecimalPlaces(Rect.Top),
        right: roundToDecimalPlaces(Rect.Right),
        bottom: roundToDecimalPlaces(Rect.Bottom)
      }

      const rawContent = Module._GetAnnotContent(annotPtr)
      const content = UTF8ToString(rawContent)
      const rawCreateDate = Module._GetAnnotCreationDate(annotPtr)
      const createDate = parseAdobePDFTimestamp(UTF8ToString(rawCreateDate))
      console.log('content', content, createDate)

      FontDa = {}
      Module._GetAnnotFont(annotPtr)

      RGBColor = {}
      Module._GetAnnotRGBColor(annotPtr)
      for (key in RGBColor) {
        RGBColor[key] = roundToDecimalPlaces(RGBColor[key] * 255)
      }
      return {
        content,
        createDate,
        rect,
        color: {
          R: RGBColor.R,
          G: RGBColor.G,
          B: RGBColor.B
        },
        font: FontDa.fontSize
      }
    })

    messageHandler.on('GetAnnotLink', (data) => {
      const { page, annotPtr } = data
      
      Rect = {}
      Module._GetAnnotRect(page, annotPtr)
      const content = Module._GetAnnotContent(annotPtr)
      const createDate = Module._GetAnnotCreationDate(annotPtr)
      FontDa = {}
      Module._GetAnnotFont(annotPtr)
      RGBColor = {}
      Module._GetAnnotRGBColor(annotPtr)
      return {
        content,
        createDate,
        rect: Rect,
        color: RGBColor,
        font: FontDa.fontSize
      }
    })

    messageHandler.on('PushRenderTask', (data) => {
      const { page, width, height } = data
      
      let param = {
        page,
        scale: data.zoom,
        left: 0,
        right: parseInt(width),
        bottom: parseInt(height),
        top: 0,
        mode: -1,
        flag: 0,
        form: false
      }
      let imageArray = RenderPageBitmapWithMatrix(param)
      return { imageArray }
    })

    messageHandler.on('SetAnnotRect', (data) => {
      const { ptr, annotPtr, rect } = data
      const { left, right, bottom, top } = rect

      const result = Module._SetAnnotRect(ptr, annotPtr, left, right, bottom, top)
      return result
    })

    messageHandler.on('SetAnnotContent', (data) => {
      const { annotPtr, content: rawContent } = data
      
      const content = stringToNewUTF8(rawContent)
      const result = Module._SetAnnotContent(annotPtr, content)
      return result
    })

    messageHandler.on('SetAnnotRGBColor', (data) => {
      const { annotPtr, color } = data
      const { R, G, B } = color

      const result = Module._SetAnnotRGBColor(annotPtr, R, G, B)
      return result
    })

    messageHandler.on('SetAnnotTransparency', (data) => {
      const { annotPtr, transparency } = data

      const result = Module._SetAnnotTransparency(annotPtr, transparency)
      return result
    })
  }

}

async function initDoc() {
  while (!runtimeInitialized) {
    await sleep(100)
  }
  return new Promise((resolve, reject) => {
    const doc = Module._InitDocument()
    resolve(doc)
  })
}

initDoc().then((doc) => {
  CPDFWorker.setup(doc)
})

function RenderPageBitmapWithMatrix(data) {
  const { page, scale, left, right, bottom, top, mode, flag, form } = data
  let pixelNum = (right - left) * (bottom - top)
  let imageBytes = pixelNum * 4
  let imageptr = _malloc(imageBytes)
  for (var i = 0; i < pixelNum; i++) {
    Module.HEAP32[imageptr / 4 + i] = 0
  }

  Module._RenderPageBitmapWithMatrix(page, scale, left, right, bottom, top, mode, imageptr, flag, form)
  let imageArray = new Uint8Array(imageBytes)
  for (var i = 0; i < imageBytes; i += 4) {
    //bgra 转 rgba
    imageArray[i] = Module.HEAPU8[imageptr + i + 2]
    imageArray[i + 1] = Module.HEAPU8[imageptr + i + 1]
    imageArray[i + 2] = Module.HEAPU8[imageptr + i]
    imageArray[i + 3] = Module.HEAPU8[imageptr + i + 3]
  }
  _free(imageptr)

  return imageArray
}
